@@ -1024,17 +1024,6 @@ type SContinue struct {
 	Label *LocRef
 }
 
-func IsSuperCall(stmt Stmt) bool {
-	if expr, ok := stmt.Data.(*SExpr); ok {
-		if call, ok := expr.Value.Data.(*ECall); ok {
-			if _, ok := call.Target.Data.(*ESuper); ok {
-				return true
-			}
-		}
-	}
-	return false
-}
-
 type ClauseItem struct {
 	Alias string
 
